unit Disk_Test;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FileCtrl, StdCtrls, ComCtrls, Buttons;

type
  TTestDiskForm = class(TForm)
    TestDrive: TDriveComboBox;
    TestDirectory: TDirectoryListBox;
    TestFiles: TFileListBox;
    RunTestButton: TBitBtn;
    CancelButton: TBitBtn;
    MemorySize: TTrackBar;
    BufferSizeTag: TLabel;
    MessageList: TListBox;
    RunContinuous: TCheckBox;
    procedure MemorySizeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure RunTestButtonClick(Sender: TObject);
  private
    { Private declarations }
    ChunkSize:integer;
    CopyBuffer: Pointer; { buffer for copying }
    ExitLoop:boolean;
    StartTime:TDateTime;
    WriteTime:TDateTime;
    ReadTime:TDateTime;
    TotalBytesWritten:integer;
    TotalBytesRead:integer;
    procedure DoWriteFile(const FileName:string);
  public
    { Public declarations }
  end;

var
  TestDiskForm: TTestDiskForm;

function GetFileSize(const FileName: string): LongInt;

implementation

{$R *.DFM}

uses Consts,ShellAPI;

function ZeroIntToStr(i,n:integer):string;
var
   s:string;
begin
   s:=IntToStr(i);
   while length(s)<n do s:='0'+s;
   ZeroIntToStr:=s;
end;

procedure TTestDiskForm.DoWriteFile(const FileName:string);
type
   CBuffer=array[1..MaxInt div 2] of byte;
var
   BytesRead,BytesWritten: Longint;
   Dest: Integer; { handles }
   h,m,s,ms:word;
   ETime:TDateTime;
   sr:string;
   sec:extended;
   CBP:^CBuffer;
   i:integer;
begin
   try
      GetMem(CopyBuffer, ChunkSize); { allocate the buffer }
      CBP:=CopyBuffer;
      for i:=1 to ChunkSize do
      CBP^[i]:=Random(256);
      MessageList.Items.Add(' ');
      MessageList.Items.Add(' ');
      if GetFileSize(FileName)<0 then Dest:=FileCreate(FileName)
      else Dest:=-1;
      if Dest<0 then
      raise Exception.Create('Could not create file: '+FileName);

      StartTime:=Now;
      try
         repeat
            BytesWritten:=FileWrite(Dest, CopyBuffer^, ChunkSize);
            if BytesWritten>0 then
            TotalBytesWritten:=TotalBytesWritten+BytesWritten;
            if (TotalBytesWritten mod 1024*64)=0 then
            begin
               Application.ProcessMessages;
               with MessageList do
               begin
                  Items.Delete(Items.Count-1);
                  Items.Add(IntToStr(TotalBytesWritten)
                  +' bytes written...');
                  Refresh;
               end;
            end;
         until ExitLoop or (BytesWritten<ChunkSize);
      finally
         FileClose(Dest);
         WriteTime:=Now-StartTime;
         DecodeTime(WriteTime,h,m,s,ms);
         MessageList.Items.Add('Elapsed time '
         +ZeroIntToStr(m,2)+':'+ZeroIntToStr(s,2)
         +'.'+ZeroIntToStr(ms,3));
         sec:=WriteTime*24.0*60.0*60.0;
         if sec<>0.0 then Str((TotalBytesWritten/1024.0/sec):5:1,sr)
         else sr:='';
         MessageList.Items.Add(sr+' KB/sec');
      end;

      ExitLoop:=false;
      MessageList.Items.Add(' ');
      MessageList.Items.Add(' ');
      Dest:=FileOpen(FileName,fmOpenRead+fmShareDenyWrite);
      if Dest<0 then
      raise Exception.Create('Could not open file: '+FileName);

      StartTime:=Now;
      try
         repeat
            BytesRead:=FileRead(Dest, CopyBuffer^, ChunkSize);
            if BytesRead>0 then
            TotalBytesRead:=TotalBytesRead+BytesRead;
            if (TotalBytesRead mod 1024*64)=0 then
            begin
               Application.ProcessMessages;
               with MessageList do
               begin
                  Items.Delete(Items.Count-1);
                  Items.Add(IntToStr(TotalBytesRead)
                  +' bytes read...');
                  Refresh;
               end;
            end;
         until ExitLoop or (BytesRead<ChunkSize);
      finally
         FileClose(Dest);
         ReadTime:=Now-StartTime;
         DecodeTime(ReadTime,h,m,s,ms);
         MessageList.Items.Add('Elapsed time '
         +ZeroIntToStr(m,2)+':'+ZeroIntToStr(s,2)
         +'.'+ZeroIntToStr(ms,3));
         sec:=ReadTime*24.0*60.0*60.0;
         if sec<>0.0 then Str((TotalBytesRead/1024.0/sec):5:1,sr)
         else sr:='';
         MessageList.Items.Add(sr+' KB/sec');
         DeleteFile(FileName);
      end;

   finally
      FreeMem(CopyBuffer, ChunkSize); { free the buffer }
   end;
end;

{ GetFileSize function }
{
  Returns the size of the named file without opening the file.  If the file
  doesn't exist, returns -1.
}

function GetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  if FindFirst(FileName,faAnyFile,SearchRec)=0 then
    Result:=SearchRec.Size
  else
    Result:=-1;
  FindClose(SearchRec);
end;

function AddSlash(PathN:string):string;
begin
   if length(PathN)<=0 then
   AddSlash:='\'
   else if PathN[length(PathN)]<>'\' then
   AddSlash:=PathN+'\'
   else
   AddSlash:=PathN;
end;

function DelSlash(PathN:string):string;
begin
   if length(PathN)<=0 then
   DelSlash:=''
   else if PathN[length(PathN)]<>'\' then
   DelSlash:=PathN
   else
   begin
      system.delete(PathN,length(PathN),1);
      DelSlash:=PathN;
   end;
end;

function RemoveFirstPath(FirstPath,PathN:string):string;
begin
   if pos(UpperCase(FirstPath),UpperCase(PathN))=1 then
   begin
      system.delete(PathN,1,length(FirstPath));
   end;
   if pos('\',PathN)=1 then system.delete(PathN,1,1);
   RemoveFirstPath:=PathN;
end;

function ReturnDirLevel(dirLevel:integer;
   PathN:string):string;
var
   i,j,sp:integer;
   s:string;
begin
   s:='';
   j:=dirLevel;
   sp:=0;
   for i:=1 to length(PathN) do
   if (j>0) and (PathN[i]='\') then
   begin
      sp:=i;
      dec(j);
   end;
   if sp>0 then s:=system.copy(PathN,1,sp-1);
   ReturnDirLevel:=s;
end;

function ReturnPathName(ListItem:string):string;
var
   i,j:integer;
   s:string;
begin
   i:=length(ListItem);
   while (i>0) and (ListItem[i]<>' ') do
   dec(i);
   dec(i);
   while (i>0) and (ListItem[i]<>' ') do
   dec(i);
   s:='';
   if i>1 then
   s:=system.Copy(ListItem,1,i-1);
   ReturnPathName:=s;
end;

procedure TTestDiskForm.MemorySizeChange(Sender: TObject);
begin
   ChunkSize:=(1 shl MemorySize.Position)*1024;
   BufferSizeTag.Caption:='Buffer Size: '
   +IntToStr(ChunkSize div 1024)+' KB';
end;

procedure TTestDiskForm.FormCreate(Sender: TObject);
begin
   ExitLoop:=false;
   MemorySize.Position:=9;
   MemorySizeChange(Sender);
end;

procedure TTestDiskForm.CancelButtonClick(Sender: TObject);
begin
   ExitLoop:=true;
end;

procedure TTestDiskForm.RunTestButtonClick(Sender: TObject);
begin
   ExitLoop:=false;
   RunTestButton.Enabled:=false;
   CancelButton.Enabled:=true;
   repeat
      TotalBytesWritten:=0;
      TotalBytesRead:=0;
      MessageList.Items.Clear;
      MessageList.Items.Add('Starting test...');
      DoWriteFile(AddSlash(TestDirectory.Directory)
      +'Disk Speed Test File');
   until not RunContinuous.Checked;
   RunTestButton.Enabled:=true;
   CancelButton.Enabled:=false;
end;

end.
